unit GLUnit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OpenGL, StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

type TRCOptions = set of (opDoubleBuffered,opGDI,opStereo);

var
  Form1: TForm1;
var
 FMyContext:HGLRC;
implementation

{$R *.dfm}

procedure SetupPalette(DC: HDC; PFD: TPixelFormatDescriptor);
var
 nColors, I : Integer;
 lpPalette : PLogPalette;
 byRedMask,
 byGreenMask,
 byBlueMask : Byte;
 Palette : HPalette;

begin
 nColors:=1 shl Pfd.cColorBits;
 GetMem(lpPalette,SizeOf(TLogPalette)+(nColors*SizeOf(TPaletteEntry)));
 try
   lpPalette^.palVersion:=$300;
   lpPalette^.palNumEntries:=nColors;
   byRedMask :=(1 shl Pfd.cRedBits)-1;
   byGreenMask:=(1 shl Pfd.cGreenBits)-1;
   byBlueMask :=(1 shl Pfd.cBlueBits)-1;
{$ifopt R+} {$define RangeCheck} {$endif} {$R-}

   for I:=0 to nColors-1 do
   begin
     lpPalette^.palPalEntry[I].peRed :=(((I shr Pfd.cRedShift) and byRedMask) *255) DIV byRedMask;
     lpPalette^.palPalEntry[I].peGreen:=(((I shr Pfd.cGreenShift) and byGreenMask)*255) DIV byGreenMask;
     lpPalette^.palPalEntry[I].peBlue :=(((I shr Pfd.cBlueShift) and byBlueMask) *255) DIV byBlueMask;
     lpPalette^.palPalEntry[I].peFlags:=0;
   end;

{$ifdef RangeCheck} {$R+} {$undef RangeCheck} {$endif}

   Palette:=CreatePalette(lpPalette^);
   if (Palette <> 0) then
   begin
     SelectPalette(DC,Palette,False);
     RealizePalette(DC);
   end;
   finally
   FreeMem(lpPalette);
   end;
end;


function CreateRenderingContext(DC: HDC; Options: TRCOptions; StencilBits: Byte): HGLRC;

var
 PFDescriptor : TPixelFormatDescriptor;
 PixelFormat : Integer;
 sp:tagPIXELFORMATDESCRIPTOR;
begin
 FillChar(PFDescriptor,SizeOf(PFDescriptor),0);
 with PFDescriptor do
   begin
     nSize:=sizeof(PFDescriptor);
     nVersion:=1;
     dwFlags:=PFD_SUPPORT_OPENGL;
     if GetObjectType(DC) = OBJ_MEMDC then
        dwFlags:=dwFlags or PFD_DRAW_TO_BITMAP
     else dwFlags:=dwFlags or PFD_DRAW_TO_WINDOW;
     if opDoubleBuffered in Options then
        dwFlags:=dwFlags or PFD_DOUBLEBUFFER;
     if opGDI in Options then
        dwFlags:=dwFlags or PFD_SUPPORT_GDI;
     if opStereo in Options then
        dwFlags:=dwFlags or PFD_STEREO;
     iPixelType:=PFD_TYPE_RGBA;
     cColorBits:=32;
     cDepthBits:=32;
     cStencilBits:=StencilBits;
     iLayerType:=Byte(PFD_MAIN_PLANE);
   end;

    PixelFormat:=ChoosePixelFormat(DC,@PFDescriptor);
   SetPixelFormat(DC,PixelFormat,@PFDescriptor);
// check the properties just set
  sp:=PFDescriptor;
  DescribePixelFormat(DC,PixelFormat,SizeOf(PFDescriptor),sp);

   with PFDescriptor do
    if (dwFlags and PFD_NEED_PALETTE) <> 0 then
       SetupPalette(DC,PFDescriptor);
  Result:=wglCreateContext(DC);
 end;


procedure TForm1.FormCreate(Sender: TObject);
begin
 FMyContext:=CreateRenderingContext(Canvas.Handle,[],0);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
 wglMakeCurrent(Canvas.Handle,FMyContext);
 glViewport(0,0,Width,Height);
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity;
 gluPerspective(30,Width/Height,1,100);
 wglMakeCurrent(0,0);
 Refresh;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 // activate the RC
 wglMakeCurrent(Canvas.Handle,FMyContext);

// clear color and depth buffer
 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

// initialize transformation pipeline
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity;
 
 glBegin(GL_POLYGON);
  {glTexCoord2f(0,0);}
  glVertex3f(0,0,0);
  {glTexCoord2f(1,0);}
  glVertex3f(1,0,0);
  {glTexCoord2f(1,1);}
  glVertex3f(1,1,0);
  {glTexCoord2f(0,1);}
  glVertex3f(0,1,0);
 glEnd;

 wglMakeCurrent(0,0);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
 glBegin(GL_POLYGON);
  glTexCoord2f(0,0);
  glVertex3f(0,0,0);
  glTexCoord2f(1,0);
  glVertex3f(1,0,0);
  glTexCoord2f(1,1);
  glVertex3f(1,1,0);
  glTexCoord2f(0,1);
  glVertex3f(0,1,0);
 glEnd;
end;

end.
