procedure SetupPalette(DC: HDC; PFD: TPixelFormatDescriptor);
var
 nColors, I : Integer;
 lpPalette : PLogPalette;
 byRedMask,
 byGreenMask,
 byBlueMask : Byte;
 Palette : HPalette;

begin
 nColors:=1 shl Pfd.cColorBits;
 GetMem(lpPalette,SizeOf(TLogPalette)+(nColors*SizeOf(TPaletteEntry)));
 try
   lpPalette^.palVersion:=$300;
   lpPalette^.palNumEntries:=nColors;
   byRedMask :=(1 shl Pfd.cRedBits)-1;
   byGreenMask:=(1 shl Pfd.cGreenBits)-1;
   byBlueMask :=(1 shl Pfd.cBlueBits)-1;
{$ifopt R+} {$define RangeCheck} {$endif} {$R-}

   for I:=0 to nColors-1 do
   begin 
     lpPalette^.palPalEntry[I].peRed :=(((I shr Pfd.cRedShift) and byRedMask) *255) DIV byRedMask;
     lpPalette^.palPalEntry[I].peGreen:=(((I shr Pfd.cGreenShift) and byGreenMask)*255) DIV byGreenMask;
     lpPalette^.palPalEntry[I].peBlue :=(((I shr Pfd.cBlueShift) and byBlueMask) *255) DIV byBlueMask;
     lpPalette^.palPalEntry[I].peFlags:=0;
   end;

{$ifdef RangeCheck} {$R+} {$undef RangeCheck} {$endif}

   Palette:=CreatePalette(lpPalette^);
   if (Palette <> 0) then
   begin
     SelectPalette(DC,Palette,False);
     RealizePalette(DC);
   end;
   finally
   FreeMem(lpPalette);
   end;
end;

type TRCOptions = set of (opDoubleBuffered,opGDI,opStereo);

function CreateRenderingContext(DC: HDC; Options: TRCOptions; StencilBits: Byte): HGLRC;

var
 PFDescriptor : TPixelFormatDescriptor;
 PixelFormat : Integer;

begin
 FillChar(PFDescriptor,SizeOf(PFDescriptor),0);
 with PFDescriptor do
   begin
     nSize:=sizeof(PFDescriptor);
     nVersion:=1;
     dwFlags:=PFD_SUPPORT_OPENGL;
     if GetObjectType(DC) = OBJ_MEMDC then
        dwFlags:=dwFlags or PFD_DRAW_TO_BITMAP
     else dwFlags:=dwFlags or PFD_DRAW_TO_WINDOW;
     if opDoubleBuffered in Options then
        dwFlags:=dwFlags or PFD_DOUBLEBUFFER;
     if opGDI in Options then
        dwFlags:=dwFlags or PFD_SUPPORT_GDI;
     if opStereo in Options then
        dwFlags:=dwFlags or PFD_STEREO;
     iPixelType:=PFD_TYPE_RGBA;
     cColorBits:=32;
     cDepthBits:=32;
     cStencilBits:=StencilBits;
     iLayerType:=Byte(PFD_MAIN_PLANE);
   end;

    PixelFormat:=ChoosePixelFormat(DC,@PFDescriptor);
   SetPixelFormat(DC,PixelFormat,@PFDescriptor);
// check the properties just set
  DescribePixelFormat(DC,PixelFormat,SizeOf(PFDescriptor),@PFDescriptor);

   with PFDescriptor do
    if (dwFlags and PFD_NEED_PALETTE) <> 0 then   
       SetupPalette(DC,PFDescriptor);
  Result:=wglCreateContext(DC);
 end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 FMyContext:=CreateRenderingContext(Canvas.Handle,[opDoubleBuffer],0);
end;

