unit Render;

interface

uses
  Windows,
  SysUtils,
  OpenGL,
  Main;

const
 N=40;
 N1=20;
 R1:Single=2;
 R2:Single=0.4;

type
 T_XYZ=record
  X,Y,Z:Single
 end;
 TSurface=array[1..N+1,1..N+1] of T_XYZ;

procedure InitOpenGL;
procedure DoneOpenGL;
procedure RenderGL;
procedure InitSurface;
procedure DrawOrts;
procedure DrawOne(const i,j:Integer);
procedure InitLifeArea;
function GetNeighbors(const i,j:Integer):Byte;
procedure NextLifeArea;
procedure GetLight(const x1,y1,z1:Single; const x2,y2,z2:Single; var c:Single);
procedure SetLight(const AV,AH:Single);


implementation

var
  glDC: HDC;
  GLContext: HGLRC;
  OpenGLReady: boolean;
  Area,nArea:Array[1..N,1..N] of Byte;
  Toroid:TSurface;
  lx,ly,lz:Single;
  Count:DWord;
  DCount:ShortInt;

procedure Error(Msg:string;Num:integer);
begin
  Msg := Msg+' '+IntToStr(Num);
  MessageBox(0,PChar(Msg),nil,MB_OK);
  PostQuitMessage(0);
end;

procedure InitOpenGL;
var
  pfd: TPixelFormatDescriptor;
  FormatIndex,
  dx,
  dy: integer;
begin
  fillchar(pfd,SizeOf(pfd),0);
  OpenGLReady := false;
  with pfd do
  begin
    nSize      := SizeOf(pfd);
    nVersion   := 1; {The current version of the desccriptor is 1}
    dwFlags    := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER;
    iPixelType := PFD_TYPE_RGBA;
    cColorBits := 24; {support 24-bit color}
    cDepthBits := 32; {depth of z-axis}
    iLayerType := PFD_MAIN_PLANE;
  end; {with}
  glDC := getDC(MainWindow);
  FormatIndex := ChoosePixelFormat(glDC,@pfd);
  if FormatIndex=0 then
    Error('ChoosePixelFormat failed ',GetLastError);

  if not SetPixelFormat(glDC,FormatIndex,@pfd) then
    Error('SetPixelFormat failed ',GetLastError);

  GLContext := wglCreateContext(glDC);
  if GLContext=0 then
    Error('wglCreateContext failed ',GetLastError);

  if not wglMakeCurrent(glDC,GLContext) then
    Error('wglMakeCurrent failed ',GetLastError);

  glMatrixMode(GL_PROJECTION);
  glFrustum(-2.0,2.0,-2.0,2.0,2.0,37.0);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;

  dx:= GetSystemMetrics(SM_CXSCREEN);
  dy:= GetSystemMetrics(SM_CYSCREEN);
  glViewPort(0,0,dx,dy);

  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
  glClearColor(0.0,0.0,0.0,1.0);
  OpenGLReady := true;
  Count:=3;
  DCount:=1;
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
end;

procedure DoneOpenGL;
begin
  OpenGLReady := false;
  wglMakeCurrent(glDC,0);
  wglDeleteContext(GLContext);
end;

Procedure DrawSurface;
var
 i,j:Integer;
 dX1,dY1,dZ1:Single;
 dX2,dY2,dZ2:Single;
 C:Single;
begin
 for i:=1 to N do
  for j:=1 to N do
  begin
   dX1:=Toroid[i+1,j].X - Toroid[i,j].X;
   dY1:=Toroid[i+1,j].Y - Toroid[i,j].Y;
   dZ1:=Toroid[i+1,j].Z - Toroid[i,j].Z;
   dX2:=Toroid[i,j+1].X - Toroid[i,j].X;
   dY2:=Toroid[i,j+1].Y - Toroid[i,j].Y;
   dZ2:=Toroid[i,j+1].Z - Toroid[i,j].Z;
   GetLight(dX1,dY1,dZ1,dX2,dY2,dZ2,C);
   glColor3f(C,C*Count/59,SQRT(C));
   if Area[i,j]=1 then
    glColor3f(C,0,Count/59);
   glBegin(GL_LINE_LOOP);
    glVertex3f(Toroid[i,j].X,Toroid[i,j].Z,Toroid[i,j].Y);
    glVertex3f(Toroid[i,j+1].X,Toroid[i,j+1].Z,Toroid[i,j+1].Y);
    glVertex3f(Toroid[i+1,j+1].X,Toroid[i+1,j+1].Z,Toroid[i+1,j+1].Y);
    glVertex3f(Toroid[i+1,j].X,Toroid[i+1,j].Z,Toroid[i+1,j].Y);
   glEnd;
   glBegin(GL_POLYGON);
    glVertex3f(Toroid[i,j].X,Toroid[i,j].Z,Toroid[i,j].Y);
    glVertex3f(Toroid[i,j+1].X,Toroid[i,j+1].Z,Toroid[i,j+1].Y);
    glVertex3f(Toroid[i+1,j+1].X,Toroid[i+1,j+1].Z,Toroid[i+1,j+1].Y);
    glVertex3f(Toroid[i+1,j].X,Toroid[i+1,j].Z,Toroid[i+1,j].Y);
   glEnd;
  end;
end;

procedure DrawOrts;
begin
 glBegin(GL_LINES);
  glColor3f(1,0,0);
  glVertex3f(0,0,0);
  glVertex3f(0,0,1);
  glColor3f(0,1,0);
  glVertex3f(0,0,0);
  glVertex3f(0,1,0);
  glColor3f(0,0,1);
  glVertex3f(0,0,0);
  glVertex3f(1,0,0);
 glEnd;
end;

procedure DrawOne;
begin
 glBegin(GL_POLYGON);
  glColor3f(1,0,0);
  glVertex3f(Toroid[i,j].X,Toroid[i,j].Z,Toroid[i,j].Y);
  glVertex3f(Toroid[i,j+1].X,Toroid[i,j+1].Z,Toroid[i,j+1].Y);
  glVertex3f(Toroid[i+1,j+1].X,Toroid[i+1,j+1].Z,Toroid[i+1,j+1].Y);
  glVertex3f(Toroid[i+1,j].X,Toroid[i+1,j].Z,Toroid[i+1,j].Y);
 glEnd;
end;

function GetNeighbors;
var
 S:Byte;
 i1,i2,j1,j2:Integer;
begin
 {Encyrcling i coordinates}
 if i>1 then
  i1:=i-1
 else
  i1:=N;
 if i<N then
  i2:=i+1
 else
  i2:=1;
 {Encyrcling j coordinates}
 if j>1 then
  j1:=j-1
 else
  j1:=N;
 if j<N then
  j2:=j+1
 else
  j2:=1;
 {Evaluating the Sum}
 S:=Area[i1,j1] + Area[i1,j] + Area[i1,j2]
   +Area[i,j1]  +            + Area[i,j2]
   +Area[i2,j1] + Area[i2,j] + Area[i2,j2];
 GetNeighbors:=S;
end;


procedure RenderGL;
begin
 if not openGLReady then
   exit;
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
   glPushMatrix;
    glTranslatef(0.0,0.0,-5.3);
    glRotatef(6*Count,1,0,0);
    glRotatef(4.2549*Count,0,0,1);
     glPushMatrix;
      DrawSurface;
      DrawOrts;
     glPopMatrix;
   glPopMatrix;
  SwapBuffers(glDC);
 NextLifeArea;
 Inc(Count,DCount);
 if ((Count>59) or (Count<2)) then
  DCount:=-1*DCount;
 SetLight(6.89*Count,271*Sin(2*pi*Count/60));
end;

procedure InitSurface;
var
 i,j:Integer;
 A1,A2:Single;
begin
 Randomize;
 lx:=0.3; ly:=0.6; lz:=1;
 lx:=lx/Sqrt(SQR(lx) + SQR(ly) + SQR(lz));
 ly:=ly/Sqrt(SQR(lx) + SQR(ly) + SQR(lz));
 lz:=lz/Sqrt(SQR(lx) + SQR(ly) + SQR(lz));
 for i:=1 to N do
  for j:=1 to N do
  begin
   A1:=2*pi*i/N;
   A2:=2*pi*j/N;
   Toroid[i,j].Z:=R2*Sin(A2);
   Toroid[i,j].X:=(R1 + R2*Cos(A2))*Cos(A1);
   Toroid[i,j].Y:=(R1 + R2*Cos(A2))*Sin(A1)
  end;
 for i:=1 to N+1 do
  Toroid[i,N+1]:=Toroid[i,1];
 for j:=1 to N+1 do
  Toroid[N+1,j]:=Toroid[1,j];
end;

procedure InitLifeArea;
var
 i,j:Integer;
begin
 for i:=1 to N do
  for j:=1 to N do
   Area[i,j]:=0;
 for i:=7 to 13 do
  for j:=7 to 13 do
   Area[i,j]:=1;
end;

procedure NextLifeArea;
var
 i,j:Integer;
 S:Byte;
begin
 for i:=1 to N do
  for j:=1 to N do
  begin
   S:=GetNeighbors(i,j);
   nArea[i,j]:=Area[i,j];
   if ((S<2) or (S>4)) then
    nArea[i,j]:=0;
   if S=3 then
    nArea[i,j]:=1;
  end;
 for i:=1 to N do
  for j:=1 to N do
  begin
   Area[i,j]:=nArea[i,j];
  end;
end;

procedure GetLight;
var
 xr,yr,zr:Single;
 l:Single;
begin
 xr:=y1*z2 - z1*y2;
 yr:=z1*x2 - z2*x1;
 zr:=x1*y2 - y1*x2;
 l:=Sqrt(xr*xr + yr*yr + zr*zr);
 xr:=xr/l;
 yr:=yr/l;
 zr:=zr/l;
 c:=lx*xr + ly*yr + lz*zr;
 c:=0.5 + c/2;
end;

procedure SetLight;
var
 AV1,AH1:Single;
begin
 AV1:=pi*AV/180;
 AH1:=pi*AH/180;
 lx:=Cos(AV1)*Cos(AH1);
 ly:=Cos(AV1)*Sin(AH1);
 lz:=Sin(AV1);
end;

end.
